// main.ts
import { McpServer } from "@modelcontextprotocol/sdk/server/mcp.js";
import { StdioServerTransport } from "@modelcontextprotocol/sdk/server/stdio.js";
import { x } from "tinyexec";
import path from "node:path";

// tools.ts
import { z } from "zod";
var tools = {
  speed_up: {
    name: "speed_up",
    description: "Speed up a video",
    input: {
      input_file: z.string().describe("Path to input file"),
      output_file: z.string().describe("Path to output file, output to the same directory if not specified").optional(),
      max_fps: z.number().min(1).max(60).default(30).describe("Max FPS for the output file"),
      speed_factor: z.number().min(0.1).max(10).default(2).describe("Speed factor for the output file, default to 2x sped up")
    }
  },
  extract_audio: {
    name: "extract_audio",
    description: "Extract audio as mp3 from a video",
    input: {
      input_file: z.string().describe("Path to input file"),
      output_file: z.string().describe("Path to output file, output to the same directory if not specified").optional()
    }
  },
  compress_video: {
    name: "compress_video",
    description: "Compress a video with configurable quality",
    input: {
      input_file: z.string().describe("Path to input file"),
      output_file: z.string().describe("Path to output file, output to the same directory if not specified").optional(),
      quality: z.number().min(20).max(51).default(28).describe("CRF value (20-51, higher means smaller file size but lower quality, 28 is recommended for good compression)"),
      preset: z.enum([
        "ultrafast",
        "superfast",
        "veryfast",
        "faster",
        "fast",
        "medium",
        "slow",
        "slower",
        "veryslow"
      ]).default("medium").describe("Compression preset (slower = better compression)"),
      scale: z.number().min(-1).max(4320).default(-1).describe("Output height in pixels (-1 means keep original size)"),
      maxrate: z.string().default("1500k").describe("Maximum bitrate (e.g. '1000k', '2M')"),
      bufsize: z.string().default("3000k").describe("Buffer size (should be about double the maxrate)")
    }
  }
};

// main.ts
var ffmpeg = process.env.FFMPEG_PATH || "ffmpeg";
var sanitizeFilename = (filename) => {
  return filename.replace(/[^\w\s-\.]/g, "");
};
var getOutputFilePath = (input, ext) => {
  const input_dir = path.dirname(input);
  const input_filename = path.basename(input, path.extname(input));
  const sanitized_filename = sanitizeFilename(input_filename);
  return path.join(input_dir, `${sanitized_filename}_compressed${ext}`);
};
var server = new McpServer({
  name: "my-ffmpeg-mcp",
  version: "1.0.0"
}, {
  capabilities: {
    logging: {}
  }
});
server.tool(tools.speed_up.name, tools.speed_up.description, tools.speed_up.input, async (args) => {
  const output_file = args.output_file || getOutputFilePath(args.input_file, path.extname(args.input_file));
  const result = x(ffmpeg, [
    `-i`,
    args.input_file,
    `-filter:v`,
    `setpts=${1 / args.speed_factor}*PTS,fps=fps=${args.max_fps}`,
    `-af`,
    `atempo=${args.speed_factor}`,
    `-y`,
    output_file
  ]);
  let output = "";
  for await (const line of result) {
    output += line;
    await server.server.sendLoggingMessage({
      level: "debug",
      data: {
        line
      }
    });
  }
  if (result.exitCode !== 0) {
    return {
      content: [
        {
          type: "text",
          text: `Error: ${output}`
        }
      ],
      isError: true
    };
  }
  return {
    content: [
      {
        type: "text",
        text: `Successfully sped up video to: ${output_file}`
      }
    ]
  };
});
server.tool(tools.extract_audio.name, tools.extract_audio.description, tools.extract_audio.input, async (args) => {
  const output_file = args.output_file || getOutputFilePath(args.input_file, ".mp3");
  const result = await x(ffmpeg, [
    `-i`,
    args.input_file,
    `-vn`,
    `-acodec`,
    `mp3`,
    `-y`,
    output_file
  ]);
  if (result.exitCode !== 0) {
    return {
      content: [
        {
          type: "text",
          text: `Error: ${result.stderr}`
        }
      ],
      isError: true
    };
  }
  return {
    content: [
      {
        type: "text",
        text: `Successfully extracted audio to: ${output_file}`
      }
    ]
  };
});
server.tool(tools.compress_video.name, tools.compress_video.description, tools.compress_video.input, async (args) => {
  const output_file = args.output_file || getOutputFilePath(args.input_file, path.extname(args.input_file));
  const scaleFilter = args.scale !== -1 ? `-vf scale=-1:${args.scale}` : "";
  const result = x(ffmpeg, [
    `-i`,
    args.input_file,
    `-c:v`,
    `libx264`,
    `-crf`,
    args.quality.toString(),
    `-preset`,
    args.preset,
    `-maxrate`,
    args.maxrate,
    `-bufsize`,
    args.bufsize,
    ...scaleFilter ? [scaleFilter] : [],
    `-c:a`,
    `aac`,
    `-b:a`,
    `96k`,
    `-movflags`,
    `+faststart`,
    `-y`,
    output_file
  ]);
  let output = "";
  for await (const line of result) {
    output += line;
    await server.server.sendLoggingMessage({
      level: "debug",
      data: {
        line
      }
    });
  }
  if (result.exitCode !== 0) {
    return {
      content: [
        {
          type: "text",
          text: `Error: ${output}`
        }
      ],
      isError: true
    };
  }
  return {
    content: [
      {
        type: "text",
        text: `Successfully compressed video to: ${output_file}`
      }
    ]
  };
});
async function main() {
  const transport = new StdioServerTransport;
  await server.connect(transport);
}
main().catch((err) => {
  console.error("Error:", err);
  process.exit(1);
});
